/*
** keycodes v1.0
**
** Show curses keycodes
**
** Copyright (C) 14.5.96 by Andreas Ley <andreas.ley@kit.edu>
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
**
** This program has been tested on a HP9000/715 with HP-UX A.09.07
** In this environment, neither lint -u nor gcc -Wall -fsyntax-only produce
** any messages. If you encounter any errors or need to make any changes to
** port it to another platform, please contact me.
**
** Version history
**
** Version 1.0 - 14.5.96
**	Initial version
*/

static char copyright[] = "@(#)Copyright (C) 1996 by Andreas Ley (andreas.ley@kit.edu)";
static char sccsid[] = "@(#)keycodes v1.0 - Show curses keycodes";


#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <signal.h>

#ifdef ultrix
#include <cursesX.h>
#else
#ifdef linux
#include <ncurses.h>
#else
#include <curses.h>
#endif
#endif


#ifndef FALSE
#define	FALSE	(0)		/* This is the naked Truth */
#define	TRUE	(1)		/* and this is the Light */
#endif


#define	checkkey(key,keycode,keysym)	if (key==keycode) printw(" = %s",keysym)

char	*image;
int	debug=0;


static void handler()
{
	endwin();
	exit(0);
}



static void usage()
{
	(void)fprintf(stderr,"Usage: %s [-h] [-v]\n",image);
	exit(1);
}


int main(argc,argv)
int	argc;
char	*argv[];
{
	int		c,n;
	extern char	*optarg;
	extern int	optind;
	char		buffer[16];

	if ((image=strrchr(argv[0],'/')))
		image++;
	else
		image=argv[0];

	while ((c=getopt(argc,argv,"Dvh?"))!=EOF)
		switch ((char)c) {
		case 'D':
			debug++;
			break;
		case 'v':
			(void)fprintf(stderr,"%s\n",sccsid+4);
			(void)fprintf(stderr,"%s\n",copyright+4);
			exit(0);
		case 'h':
			(void)fprintf(stderr,"%s\n",sccsid+4);
			(void)fprintf(stderr,"%s\n",copyright+4);
		case '?':
			usage();
		}

	if (optind<argc)
		usage();

	(void)signal(SIGHUP,handler);
        (void)signal(SIGINT,handler);
        (void)signal(SIGQUIT,handler);
        (void)signal(SIGPIPE,handler);
        (void)signal(SIGALRM,handler);
        (void)signal(SIGTERM,handler);
        (void)signal(SIGPWR,handler);

	if (debug)
		(void)fprintf(stderr,"[%d] initscr()\n",__LINE__);
	initscr();
	if (debug)
		(void)fprintf(stderr,"[%d] nonl()\n",__LINE__);
	nonl();
	if (debug)
		(void)fprintf(stderr,"[%d] noecho()\n",__LINE__);
	noecho();
	if (debug)
		(void)fprintf(stderr,"[%d] cbreak()\n",__LINE__);
	cbreak();
	if (debug)
		(void)fprintf(stderr,"[%d] keypad()\n",__LINE__);
	keypad(stdscr,TRUE);
	if (debug)
		(void)fprintf(stderr,"[%d] scrollok()\n",__LINE__);
	scrollok(stdscr,TRUE);
	if (debug)
		(void)fprintf(stderr,"[%d] printw()\n",__LINE__);
	printw("%s\n",sccsid+4);
	printw("%s\n\n",copyright+4);
	if (debug)
		(void)fprintf(stderr,"[%d] refresh()\n",__LINE__);
	refresh();

	for (;;) {
		c=getch();
		printw("getch() = %4d = %#5x = %#5o",c,c,c);
		if (32<=c&&c<=126||160<=c&&c<=255)
			printw(" = '%c'",c);
		checkkey(c,KEY_BREAK,"KEY_BREAK");
		checkkey(c,KEY_DOWN,"KEY_DOWN");
		checkkey(c,KEY_UP,"KEY_UP");
		checkkey(c,KEY_LEFT,"KEY_LEFT");
		checkkey(c,KEY_RIGHT,"KEY_RIGHT");
		checkkey(c,KEY_HOME,"KEY_HOME");
		checkkey(c,KEY_BACKSPACE,"KEY_BACKSPACE");
		for (n=0;n<64;n++) {
			sprintf(buffer,"KEY_F(%d)",n);
			checkkey(c,KEY_F(n),buffer);
		}
		checkkey(c,KEY_DL,"KEY_DL");
		checkkey(c,KEY_IL,"KEY_IL");
		checkkey(c,KEY_DC,"KEY_DC");
		checkkey(c,KEY_IC,"KEY_IC");
		checkkey(c,KEY_EIC,"KEY_EIC");
		checkkey(c,KEY_CLEAR,"KEY_CLEAR");
		checkkey(c,KEY_EOS,"KEY_EOS");
		checkkey(c,KEY_EOL,"KEY_EOL");
		checkkey(c,KEY_SF,"KEY_SF");
		checkkey(c,KEY_SR,"KEY_SR");
		checkkey(c,KEY_NPAGE,"KEY_NPAGE");
		checkkey(c,KEY_PPAGE,"KEY_PPAGE");
		checkkey(c,KEY_STAB,"KEY_STAB");
		checkkey(c,KEY_CTAB,"KEY_CTAB");
		checkkey(c,KEY_CATAB,"KEY_CATAB");
		checkkey(c,KEY_ENTER,"KEY_ENTER");
		checkkey(c,KEY_SRESET,"KEY_SRESET");
		checkkey(c,KEY_RESET,"KEY_RESET");
		checkkey(c,KEY_PRINT,"KEY_PRINT");
		checkkey(c,KEY_LL,"KEY_LL");
		printw("\n");
		refresh();
	}

	endwin();

	return(0);
}
